/* Copyright (c) 1993 Neal Becker
 * All rights reserved.
 * Permission to copy for any purpose is hereby granted
 * so long as this copyright notice remains intact.
 */

/* epsy.y,v 1.3 1993/10/02 03:23:13 neal Exp */

%{
#include "param.h"
#include <stdlib.h> /* for atoi, etc. */
extern char* PrinterName;
extern param PrintParam;
extern int yylex();
extern void yyerror( char* );
extern int FoundPrinter;
static char* expand( char* in );
static char* cat( char* a, char* b );
%}

%token STRING ENGINE VPINS DPIH DPIV GRAPHICSMODE VSKIP VINC VINTERLEAVE
       XOFF YOFF WIDTH HEIGHT INITSTRING RESETSTRING HINC
       USEESCF0 USETABS USESPACES MINSKIPSPACES VSPREAD VADDVSTR

%type <strval> STRING strings pname engine initstring resetstring graphicsmode vaddvstr
%type <pval> params
%type <ival> vpins dpih dpiv vskip vinc vinterleave xoff yoff pnames hinc minskipspaces vspread
%type <dval> width height
%union {
  char* strval;
  param* pval; /* would be nice to use a struct instead of a pointer */
	       /* but can't be a member of a union */
  int ival;
  double dval;
}

%%

entries	:	entry
	|	entries entry
	;

entry	:	pnames '{' params '}' {
		  if( $1 ) {
		    PrintParam = * $3;
		    delete $3;
		    FoundPrinter++;
		    YYACCEPT;
		  }
		  else
		    delete $3;
		}
	;

pnames	:	pname { $$ = ( strcmp( $1, PrinterName ) == 0 ) ? 1 : 0; }
	| 	pnames '|' pname { 
		  if( $1 ) $$ = 1;
		  else $$ = ( strcmp( $3, PrinterName ) == 0 ) ? 1 : 0;
		}
	;

pname	:	STRING
	;

params	:	/* empty */	{ $$ = new param() ; }
	|	params engine	{ $$->Engine = $2; }
	|	params vpins	{ $$->VPins = $2; }
	|	params dpih	{ $$->DPIh = $2; }
	|	params dpiv	{ $$->DPIv = $2; }
	|	params graphicsmode { $$->GraphicsMode = $2; }
	|	params vskip	{ $$->VSkip = $2; }
	|	params vinc	{ $$->VInc = $2; }
	|	params hinc	{ $$->HInc = $2; }
	|	params vinterleave { $$->VInterleave = $2; }
	|	params xoff { $$->xoff = $2; }
	|	params yoff { $$->yoff = $2; }
	|	params width { $$->Width = $2; }
	|	params height { $$->Height = $2; }
	|	params initstring { $$->InitString = $2; }
	|	params resetstring { $$->ResetString = $2; }
	|	params vaddvstr { $$->VaddvStr = $2; }
	|	params USEESCF0 { $$->SpaceType = param::UseEscF0; }
	|	params USETABS { $$->SpaceType = param::UseTabs; }
	|	params USESPACES { $$->SpaceType = param::UseSpaces; }
	|	params minskipspaces { $$->MinSkipSpaces = $2; }
	|	params vspread { $$->VSpread = $2; }
	;

engine	:	ENGINE '=' STRING { $$ = $3; }
	;

vpins	:	VPINS '=' STRING { $$ = atoi( $3 ); }
	;

dpih	:	DPIH '=' STRING { $$ = atoi( $3 ); }
	;

dpiv	:	DPIV '=' STRING { $$ = atoi( $3 ); }
	;

graphicsmode	:	GRAPHICSMODE '=' strings
			{ $$ = expand( $3 ); free( $3 ); }
	;

vskip	:	VSKIP '=' STRING { $$ = atoi( $3 ); }
	;

vinc	:	VINC '=' STRING { $$ = atoi( $3 ); }
	;

hinc	:	HINC '=' STRING { $$ = atoi( $3 ); }
	;

vinterleave	:	VINTERLEAVE '=' STRING 
			{ $$ = atoi( $3 ); }
	;

xoff	:	XOFF '=' STRING { $$ = atoi( $3 ); }
	;

yoff	:	YOFF '=' STRING { $$ = atoi( $3 ); }
	;

width	:	WIDTH '=' STRING { $$ = atof( $3 ); }
	;

height	:	HEIGHT '=' STRING { $$ = atof( $3 ); }
	;

minskipspaces	:	MINSKIPSPACES '=' STRING { $$ = atoi( $3 ); }
	;

vspread	:	VSPREAD '=' STRING { $$ = atoi( $3 ); }
	;

initstring :	INITSTRING '=' strings { $$ = expand( $3 ); free( $3 ); }
	;

resetstring :	RESETSTRING '=' strings { $$ = expand( $3 ); free( $3 ); }
	;

vaddvstr :	VADDVSTR '=' strings { $$ = expand( $3 ); free( $3 ); }
	;

strings	: 	STRING
	|	strings STRING { $$ = cat( $1, $2 ); }
	;
%%
#include <ctype.h>
#include <string.h>

/* expand backslash escape sequences \e and \x */
static char* expand( char* in ) {
  char temp[ 256 ];		/* No, I'm not going to check. */
  char* out;
  
  for( out = temp; *in != '\0'; ) {
    if( *in != '\\' )
      *out++ = *in++;
    else {
      ++in;			/* skip over the \ */
      switch( *in ) {
      case 'e' :		/* \e -> ESC */
	*out++ = '\x1b';
	++in;
	break;
      case 'f' :		/* \f -> FF */
	*out++ = '\f';
	++in;
	break;
      case 'r':			/* \r -> CR */
	*out++ = '\r';
	++in;
	break;
      case 'n':			/* \n -> LF */
	*out++ = '\n';
	++in;
	break;
      case 'x' :				    /* \x */
	++in;
	{
				/* This will work for reasonable inputs.
				 It doesn't protect against stupid inputs. */
	  int i = 0;
	  while( ( isxdigit( *in ) ) ) {
	    if( isdigit (*in) )
	      i = i * 16 + (unsigned char)(*in++) - '0';
	    else
	      i = i * 16 + (unsigned char)tolower( *in++ )
		- 'a' + 10;
	  }
	  *out++ = (unsigned char)i;
	}
	break;
      case '\0':			/* Ok, thought you'd fool us, huh? */
	break;			/* should complain... */
      default:			/* Anything else just copy */
	*out++ = *in++;
      }
    }
  }
  *out = '\0';
  return strdup( temp );
}

static char* cat( char* a, char* b ) {
  char* c = (char*)malloc( strlen( a ) + strlen( b ) + 1 );
  strcpy( c, a );
  strcat( c, b );
  free( a );
  free( b );
  return c;
}
