#include "PROTO.h"
/*
 * Copyright (c) 1987, 1989 University of Maryland
 * Department of Computer Science.  All rights reserved.
 * Permission to copy for any purpose is hereby granted
 * so long as this copyright notice remains intact.
 */

#ifndef lint
static char rcsid[] = "/usr/local/src/CVS/eps/lib/seek.c,v 1.1.1.1 1993/10/02 01:50:20 neal Exp";
#endif

/*
 * SeekFile copies an input stdio file, if necessary, so that
 * it produces a stdio file on which fseek() works properly.
 * It returns NULL if this cannot be done; in that case, the
 * input file is closed (or otherwise rendered worthless).
 *
 * CopyFile copies an input file unconditionally.  (On non-Unix
 * machines, it might `accidentally' not copy it.)
 *
 * On Unix machines, this means `if the input is a pipe or tty,
 * copy it to a temporary file'.  On other systems, all stdio files
 * might happen to be seekable.
 */

#include <stdio.h>
#include "types.h"		/* for BSD_FILE_SYSTEM */
#include "seek.h"

#ifdef EASY_BUT_NOT_GOOD

FILE *CopyFile(FILE* f){ return (f); }
FILE *SeekFile(FILE* f){ return (f); }

#else

#include <errno.h>
#ifdef BSD_FILE_SYSTEM
#include <sys/param.h>		/* want MAXBSIZE */
#else
#include <sys/types.h>
#endif
#include <sys/stat.h>

/* long	lseek();
char	*malloc(); */
#include <stdlib.h> /* for malloc */
#if defined( __MSDOS__ ) && !defined( __GNUC__ )
#  include <io.h>
#else
#  include <osfcn.h> /* for lseek */
#endif

extern int errno;

/*
 * Make and return a version of `f' on which fseek works (unconditionally).
 * This code is somewhat Unix-specific.
 */
extern FILE *CopyFile(FILE *f)
{
	register int tf, n, ifd, w;
	FILE* Temp;
	register char *p, *buf;
	register int blksize;
	struct stat st;
	int e;
#if defined( __MSDOS__ ) && !defined( __GNUC__ ) 
#define BSIZE L_tmpnam
#else
#ifdef MAXBSIZE
#define BSIZE MAXBSIZE
#else
#define BSIZE BUFSIZ
#endif
#endif
	char stackbuf[BSIZE];

	/* get a read/write temp file which will vanish when closed */
	if ((Temp = MakeRWTempFile(stackbuf)) == NULL) {
	  fprintf(stderr, "MakeRWTempFile failed\n" );
		e = errno;
		(void) fclose(f);
		errno = e;
		return (NULL);
	}

	/* compute buffer size and choose buffer */
	ifd = fileno(f);
	buf = stackbuf;
	blksize = sizeof stackbuf;
#if defined( __MSDOS__ ) && !defined( __GNUC__ )
  blksize = 1024;
  buf = (char*)malloc( blksize );
#endif
	/* copy from input file to temp file */
	(void) lseek(ifd, 0L, SEEK_SET);	/* paranoia */
	tf = fileno( Temp );
	while ((n = read(ifd, p = buf, blksize)) > 0) {
		do {
			if ((w = write(tf, buf, n)) < 0) {
				(void) close(tf);
				(void) fclose(f);
				fprintf( stderr, "write error\n" );
				return (NULL);
			}
			p += w;
		} while ((n -= w) > 0);
	}
	e = errno;		/* in case n < 0 */
	if (buf != stackbuf)
		free(buf);
	if (n < 0) {
		(void) close(tf);
		(void) fclose(f);
		errno = e;
		perror("CopyFile");
		return (NULL);
	}

	/* discard the input file, and rewind and open the temporary */
	(void) fclose(f);
	(void) lseek(tf, 0L, SEEK_SET);
	errno = 0;
	return (Temp);
}

/*
 * Copy an input file, but only if necessary.
 */
extern FILE *SeekFile(FILE *f)
{
  int fd = fileno(f);

  if( ( lseek(fd, 0L, SEEK_SET ) >= 0 ) && !isatty(fd) ) 
    return f;
  else 
    return CopyFile(f);
}

#endif /* EASY_BUT_NOT_GOOD */
