/*:ts=8*/
/*****************************************************************************
 * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
 *
 * $Id: ffxqt.c,v 3.7 1994/05/02 05:36:42 mj Exp mj $
 *
 * Process incoming ffx control and data files
 *
 * With full supporting cast of busy files and locking. ;-)
 *
 *****************************************************************************
 * Copyright (C) 1990, 1993
 *  _____ _____
 * |     |___  |   Martin Junius             FIDO:      2:242/6.1
 * | | | |   | |   Republikplatz 3           Internet:  mj@sungate.fido.de
 * |_|_|_|@home|   D-52072 Aachen, Germany   Phone:     ++49-241-86931 (voice)
 *
 * This file is part of FIDOGATE.
 *
 * FIDOGATE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * FIDOGATE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with FIDOGATE; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#include "fidogate.h"
#include "getopt.h"

#include <unistd.h>



#define PROGRAM		"ffxqt"
#define VERSION		"$Revision: 3.7 $"



/*
 * Info from ffx control file
 */
typedef struct st_ffx
{
    char *name;
    Node from, to;		/* FTN addresses */
    char *cmd;			/* Command with args */
    char *in;			/* stdin file */
    char *decompr;		/* Decompressor */
} FFX;
    


/*
 * Prototypes
 */
int	create_bsy_files	P((void));
int	delete_bsy_files	P((void));
int	do_ffx			P((void));
FFX    *parse_ffx		P((char *));
int	exec_ffx		P((FFX *));

void	short_usage		P((void));
void	usage			P((void));



/*
 * List of addresses we're processing packets for
 */
Node *addr[2 * MAXADDRESS];
int naddr = 0;


/*
 * Create .bsy files for all addresses
 */
int create_bsy_files()
{
    int ret = OK;
    int i;
    
    for(i=0; i<naddr; i++)
	if(bink_bsy_create(addr[i], NOWAIT) == ERROR)
	{
	    addr[i] = NULL;
	    ret = ERROR;
	}
    
    return ret;
}



/*
 * Delete .bsy files for all addresses
 */
int delete_bsy_files()
{
    int ret = OK;
    int i;
    
    for(i=0; i<naddr; i++)
	if(addr[i])
	    if(bink_bsy_delete(addr[i]) == ERROR)
		ret = ERROR;
    
    return ret;
}



/*
 * Processs *.ffx control files in Binkley inbound directory
 */
int do_ffx()
{
    struct dirent *dir;
    DIR *dp;
    FFX *ffx;
    
    dp = opendir(cf_inbound());
    if(!dp)
    {
	log("$Unable to open inbound directory %s", cf_inbound());
	return ERROR;
    }
    
    while((dir = readdir(dp)))
	if(wildmat(dir->d_name, "f???????.ffx"))
	{
	    strncpy0(buffer, cf_inbound(), BUFFERSIZE);
	    strncat0(buffer, "/"         , BUFFERSIZE);
	    strncat0(buffer, dir->d_name , BUFFERSIZE);
	    debug(1, "ffxqt: control file %s", buffer);

	    ffx = parse_ffx(buffer);
	    if(!ffx)
		continue;
	    if(bink_bsy_test(&ffx->from))	/* Skip if busy */
	    {
		debug(3, "ffxqt: %s busy, skipping",
		      node_to_asc(&ffx->from, TRUE));
		continue;
	    }
	    
	    exec_ffx(ffx);
	}
    
    closedir(dp);

    return OK;
}



/*
 * Parse control file and read into memory
 */
#define SEP " \t\r\n"

FFX *parse_ffx(name)
    char *name;
{
    FILE *fp;
    static FFX ffx;
    char *buf, *p;
    
    if(ffx.name)	free(ffx.name);		ffx.name    = NULL;
    if(ffx.cmd)		free(ffx.cmd);		ffx.cmd     = NULL;
    if(ffx.in)		free(ffx.in);		ffx.in      = NULL;
    if(ffx.decompr)	free(ffx.decompr);	ffx.decompr = NULL;
    ffx.from.zone = ffx.from.net = ffx.from.node = ffx.from.point = 0;
    ffx.to  .zone = ffx.to  .net = ffx.to  .node = ffx.to  .point = 0;

    ffx.name = strsave(name);
    fp = fopen(ffx.name, R_MODE);
    if(!fp)
    {
	log("$ffxqt: can't open %s", ffx.name);
	return NULL;
    }
    
    while((buf = cf_getline(buffer, BUFFERSIZE, fp)))
	switch(*buf)
	{
	case 'U':
	    /* User name */
	    p = strtok(buf+2, SEP);
	    /* From node */
	    p = strtok(NULL , SEP);
	    if(p)
		asc_to_node(p, &ffx.from, FALSE);
	    /* To node */
	    p = strtok(NULL , SEP);
	    if(p)
		asc_to_node(p, &ffx.to, FALSE);
	    break;
	    
	case 'I':
	    /* data file */
	    p = strtok(buf+2, SEP);
	    if(p)
	    {
		ffx.in = xmalloc(strlen(cf_inbound())+1+strlen(p)+1);
		strcpy(ffx.in, cf_inbound());
		strcat(ffx.in, "/");
		strcat(ffx.in, p);
	    }
	    /* decompressor */
	    p = strtok(NULL , SEP);
	    if(p)
		ffx.decompr = strsave(p);
	    break;
	    
	case 'C':
	    ffx.cmd = strsave(buf + 2);
	    break;
	}
	
    fclose(fp);

    if(!ffx.in || !ffx.cmd)
	return NULL;
    
    debug(3, "ffx: %s", ffx.name);
    debug(3, "     %s -> %s",
	  node_to_asc(&ffx.from, TRUE), node_to_asc(&ffx.to, TRUE));
    debug(3, "     %s %s", ffx.decompr ? ffx.decompr : "", ffx.in);
    debug(3, "     %s", ffx.cmd);

    return &ffx;
}



/*
 * Execute command in ffx
 */
int exec_ffx(ffx)
    FFX *ffx;
{
    int ret;
    
    if(ffx->decompr)			/* Data file with compression */
	sprintf(buffer, "%s/%s-dir/%s %s | %s/%s-dir/%s",
		cf_libdir(), PROGRAM, ffx->decompr, ffx->in,
		cf_libdir(), PROGRAM, ffx->cmd               );
    else				/* No compression */
	sprintf(buffer, "%s/%s-dir/%s <%s",
		cf_libdir(), PROGRAM, ffx->cmd, ffx->in      );

    debug(2, "Command: %s", buffer);
    ret = (system(buffer) >> 8) & 0xff;
    debug(2, "Exit code=%d", ret);

    if(ret == 0)
    {
	unlink(ffx->name);
	unlink(ffx->in);
	return OK;
    }
    
    return ERROR;
}



/*
 * Usage messages
 */
void short_usage()
{
    fprintf(stderr, "usage: %s [-options]\n", PROGRAM);
    fprintf(stderr, "       %s --help  for more information\n", PROGRAM);
    exit(EX_USAGE);
}


void usage()
{
    fprintf(stderr, "FIDOGATE %s  %s %s\n\n",
	    version_global(), PROGRAM, version_local(VERSION) );
    
    fprintf(stderr, "usage:   %s [-options]\n\n", PROGRAM);
    fprintf(stderr, "\
options:  -I --inbound dir             set inbound directory\n\
\n\
          -v --verbose                 more verbose\n\
	  -h --help                    this help\n\
          -c --config name             read config file (\"\" = none)\n\
	  -L --lib-dir name            set lib directory\n\
	  -S --spool-dir name          set spool directory\n\
	  -a --addr Z:N/F.P            set FTN address\n\
	  -f --fake-addr Z:N/F.P       set FTN fakenet address\n\
	  -u --uplink-addr Z:N/F.P     set FTN uplink address\n\
          -z --zone Z                  select aka for zone Z\n"  );

    exit(0);
}



/***** main() ****************************************************************/

int main(argc, argv)
    int argc;
    char *argv[];
{
    Node *node;
    int c;
    char *I_flag=NULL;
    char *c_flag=NULL;
    char *S_flag=NULL, *L_flag=NULL;
    char *a_flag=NULL, *f_flag=NULL, *u_flag=NULL;
    char *z_flag=NULL;
    
    int option_index;
    static struct option long_options[] =
    {
	{ "inbound",      1, 0, 'I'},	/* Set Binkley inbound */

	{ "verbose",      0, 0, 'v'},	/* More verbose */
	{ "help",         0, 0, 'h'},	/* Help */
	{ "config",       1, 0, 'c'},	/* Config file */
	{ "spool-dir",    1, 0, 'S'},	/* Set FIDOGATE spool directory */
	{ "lib-dir",      1, 0, 'L'},	/* Set FIDOGATE lib directory */
	{ "addr",         1, 0, 'a'},	/* Set FIDO address */
	{ "fake-addr",    1, 0, 'f'},	/* Set FIDO fakenet address */
	{ "uplink-addr",  1, 0, 'u'},	/* Set FIDO uplink address */
	{ "zone",         1, 0, 'z'},	/* Select aka for zone */
	{ 0,              0, 0, 0  }
    };

    /* Init configuration */
    cf_initialize();


    while ((c = getopt_long(argc, argv, "I:vhc:S:L:a:f:u:z:",
			    long_options, &option_index     )) != EOF)
	switch (c) {
	/***** Local options *****/
	case 'I':
	    I_flag = optarg;
	    break;
	    
	/***** Common options *****/
	case 'v':
	    verbose++;
	    break;
	case 'h':
	    usage();
	    break;
	case 'c':
	    c_flag = optarg;
	    break;
	case 'S':
	    S_flag = optarg;
	    break;
	case 'L':
	    L_flag = optarg;
	    break;
	case 'a':
	    a_flag = optarg;
	    break;
	case 'f':
	    f_flag = optarg;
	    break;
	case 'u':
	    u_flag = optarg;
	    break;
	case 'z':
	    z_flag = optarg;
	    break;
	default:
	    short_usage();
	    break;
	}

    /*
     * Read config file
     */
    if(L_flag)				/* Must set libdir beforehand */
	cf_set_libdir(L_flag);
    cf_read_config_file(c_flag);

    /*
     * Process config options
     */
    if(L_flag)
	cf_set_libdir(L_flag);
    if(S_flag)
	cf_set_spooldir(S_flag);
    if(a_flag)
	cf_set_addr(a_flag);
    if(f_flag)
	cf_set_fake(f_flag);
    if(u_flag)
	cf_set_uplink(u_flag);

    cf_debug();

    if(z_flag)
	cf_set_zone(atoi(z_flag));

    if(I_flag)
	cf_set_inbound(I_flag);

    
    /** Traverse addresses **/
    for(node=cf_addr_trav(TRUE); node; node=cf_addr_trav(FALSE))
    {
	debug(5, "node=%s", node_to_asc(node, TRUE));
	addr[naddr++] = node;
    }
    /** Create busy files, if o.k., process ff files **/
    if(create_bsy_files() == OK)
	do_ffx();
    /** Delete busy files **/
    delete_bsy_files();

    exit(0);
}
