/*:ts=8*/
/*****************************************************************************
 * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
 *
 * $Id: log.c,v 3.7 1994/05/02 05:36:42 mj Exp mj $
 *
 * Log and debug functions
 *
 *****************************************************************************
 * Copyright (C) 1990, 1993
 *  _____ _____
 * |     |___  |   Martin Junius             FIDO:      2:242/6.1
 * | | | |   | |   Republikplatz 3           Internet:  mj@sungate.fido.de
 * |_|_|_|@home|   D-52072 Aachen, Germany   Phone:     ++49-241-86931 (voice)
 *
 * This file is part of FIDOGATE.
 *
 * FIDOGATE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * FIDOGATE is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with FIDOGATE; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/

#include "fidogate.h"



int verbose = 0;			/* -v --verbose option */



/*
 * strerror()  ---  get string from sys_errlist[]
 */
char *strerror(errnum)
    int errnum;
{
    extern int sys_nerr;
    extern char *sys_errlist[];

    if (errnum > 0 && errnum < sys_nerr)
	return sys_errlist[errnum];
    return "unknown error";
}



/*
 * Log to logfile.
 *
 * If first character in format string is '$', output errno and error
 * string too.
 */
#ifdef __STDC__
void log(const char *fmt, ...)
{
    va_list args;
#ifndef FAKELOG
    FILE *logfp;
    char buf[MAXPATH];
#endif
    
    va_start(args, fmt);
#else /**!__STDC__**/
void log(va_alist)
    va_dcl
{
    va_list args;
    char *fmt;
#ifndef FAKELOG
    FILE *logfp;
    char buf[MAXPATH];
#endif
    
    va_start(args);
    fmt = va_arg(args, char *);
#endif /**__STDC__*/

#ifndef FAKELOG    
    strncpy0(buf, cf_libdir(), sizeof(buf));
    strncat0(buf, "/",         sizeof(buf));
    strncat0(buf, LOG,         sizeof(buf));

    logfp = fopen(buf, A_MODE);
    if(!logfp && !verbose)
	verbose = -1;
    else
    {
	fprintf(logfp, "%s: ", date("%d %b %y %H:%M", (long *) 0));
	vfprintf(logfp, *fmt == '$' ? fmt + 1 : fmt, args);
	if (*fmt == '$')
	    fprintf(logfp, "\n\t\terrno=%d (%s)\n", errno, strerror(errno));
	else
	    fprintf(logfp, "\n");
    }
    fclose(logfp);

    /*
     * if verbose is set, print also to stderr (without date)
     */
    if (verbose)
#else /**FAKELOG**/
    if(TRUE)
#endif /**FAKELOG**/
    {
	vfprintf(stderr, *fmt == '$' ? fmt + 1 : fmt, args);
	if (*fmt == '$')
	    fprintf(stderr, "\n\t\terrno=%d (%s)\n", errno, strerror(errno));
	else
	    fprintf(stderr, "\n");
	fflush(stderr);
    }
}



/*
 * Debug output with debug level (verbose)
 */
#ifdef __STDC__
void debug(int debug_level, const char *fmt, ...)
{
    va_list args;

    va_start(args, fmt);
#else /**!__STDC__**/
void debug(va_alist)
    va_dcl
{
    va_list args;
    char *fmt;
    int debug_level;

    va_start(args);
    debug_level = va_arg(args, int);
    fmt         = va_arg(args, char *);
#endif /**__STDC__*/

    if (debug_level <= verbose) {
	vfprintf(stderr, fmt, args);
	fprintf(stderr, "\n");
    }
}
