/*
 *	Find the network number of the iee802.3 net on eth0, as per RIP
 *	info.
 */

#include <stdio.h>
#include <stdlib.h> 
#include <sys/socket.h>
#include <linux/sockios.h>
#include <linux/ipx.h>
#include <signal.h>
#include <errno.h>

typedef struct
{
/* Novell stuff is written to 8086 style packing rules so you tend to
   get a lot of this crap... Still at least gcc allows you to tweak
   structure alignments */
	short Query1 __attribute__ ((packed));
	unsigned short Query2 __attribute__ ((packed));
} SapQueryPacket;


typedef struct
{
	short Type __attribute__ ((packed));
	short Service __attribute__ ((packed));
	char Name[48] __attribute__ ((packed));
	unsigned long Network __attribute__ ((packed));
	unsigned char Node[6] __attribute__ ((packed));
	unsigned short Socket __attribute__ ((packed));
	unsigned short HopCount __attribute__ ((packed));
} SapResponse;

volatile int expired=0;

void Expiry()
{
	expired=1;
}

void main(int argc,char *argv[])
{
	int sock;
	int size;
	SapQueryPacket r;
	struct sockaddr_ipx ipxs;
	struct ipx_route_def rt;
	int opt=1;
	int ct=0;
	int zero=0;
	SapResponse sr;

	signal(SIGALRM,Expiry);
		
	sock=socket(AF_IPX,SOCK_DGRAM,PF_IPX);
	if(sock==-1)
	{
		perror("socket");
		exit(1);
	}
	
	rt.ipx_network=htonl(0);	
	rt.ipx_router_network=IPX_ROUTE_NO_ROUTER;
	strncpy(rt.ipx_device,"eth0",16);
	rt.ipx_flags=0;
	
	if(ioctl(sock,SIOCADDRT,(void *)&rt)==-1)
	{
		perror("Allocating route '0'");
		exit(1);
	}

	opt=0;
	if(setsockopt(sock,SO_DEBUG,SOL_SOCKET,&opt,sizeof(opt))==-1)
	{
		perror("setsockopt");
		ioctl(sock,SIOCDELRT,&zero);
		exit(1);
	}

	opt=1;
	/* Permit broadcast output */
	if(setsockopt(sock,SOL_SOCKET,SO_BROADCAST, &opt,sizeof(opt))==-1)
	{
		perror("setsockopt");
		ioctl(sock,SIOCDELRT,&zero);		
		exit(1);
	}
	
	/* This is a special for IPX sockets that allows the superuser
	   to change the IPX type field of a socket */
	   
	opt=0x11;	/* Remember no htons! - its a byte */
	
	if(setsockopt(sock,SOL_SOCKET,SO_TYPE,&opt,sizeof(opt))==-1)
	{
		perror("setsockopt");
		ioctl(sock,SIOCDELRT,&zero);
		exit(1);
	}
	
	ipxs.sipx_family=AF_IPX;
	ipxs.sipx_network=htonl(0);
	ipxs.sipx_port=htons(0x452);
	
	if(bind(sock,&ipxs,sizeof(ipxs))==-1)
	{
		perror("bind");
		ioctl(sock,SIOCDELRT,&zero);
		exit(1);
	}
	
	while(ct++<10)
	{
		int sz;
			
		r.Query1=htons(3);
		r.Query2=htons(4);
		memcpy(ipxs.sipx_node,"\377\377\377\377\377\377",6);	
		if(sendto(sock,(void *)&r,sizeof(r),0,&ipxs,sizeof(ipxs))==-1)
		{
			perror("sendto");
			ioctl(sock,SIOCDELRT,&zero);
			exit(1);
		}
		
		expired=0;
		alarm(2);
	
		while(!expired)
		{
			size=recvfrom(sock,&sr,sizeof(sr),0,&ipxs,&sz);
			if(size==-1 && errno != EAGAIN)
			{
				perror("recvfrom");
				ioctl(sock,SIOCDELRT,&zero);
				exit(1);
			}
			if(htonl(ipxs.sipx_network)!=0)
			{
				printf("Network is %ld\n",htonl(ipxs.sipx_network));
				alarm(0);
				ct=10;
				break;
			}
		}
			
	}
	printf("Received size= %d\n",size);
	printf("Response type= %d\n",ntohs(sr.Type));
	printf("Nearest file server is %s.\n",sr.Name);	
	printf("File server net is %ld.\n",ntohl(sr.Network));
	ioctl(sock,SIOCDELRT,&zero);	
	rt.ipx_network=ipxs.sipx_network;
	if(ioctl(sock,SIOCADDRT,&rt)==-1)
	{
		perror("Add Real Route");
		exit(1);
	}
}

