/*
 *	A quick test program that listens to the IPX SAP response frames
 */

#include <stdio.h>
#include <stdlib.h> 
#include <sys/socket.h>
#include <linux/sockios.h>
#include <linux/ipx.h>

typedef struct
{
	short Type __attribute__ ((packed));
	char Name[48] __attribute__ ((packed));
	unsigned long Network __attribute__ ((packed));
	unsigned char Node[6] __attribute__ ((packed));
	unsigned short Socket __attribute__ ((packed));
	unsigned short HopCount __attribute__ ((packed));
} SapRecord;

void main(int argc,char *argv[])
{
	static char *SapOps[]={"Undefined","Query","Response","Find Nearest Server","Find Nearest Server Response"};
	int sock;
	int size;
	char buf[1024];
	struct sockaddr_ipx ipxs;
	int opt=1;
	
	sock=socket(AF_IPX,SOCK_DGRAM,PF_IPX);
	if(sock==-1)
	{
		perror("socket");
		exit(1);
	}
	
	if(setsockopt(sock,SOL_SOCKET,SO_DEBUG,&opt,sizeof(opt))==-1)
	{
		perror("setsockopt");
/*		exit(1);*/
	}
	
	ipxs.sipx_family=AF_IPX;
	ipxs.sipx_network=htonl(0x1B);
	ipxs.sipx_port=htons(0x452);
	
	if(bind(sock,&ipxs,sizeof(ipxs))==-1)
	{
		perror("bind");
		exit(1);
	}
	
	while(1)
	{
		int sz;
		SapRecord *s=(SapRecord *)&buf[2];
		size=recvfrom(sock,buf,sizeof(buf),0,&ipxs,&sz);
		if(size==-1)
		{
			perror("recvfrom");
			exit(1);
		}
		printf("SAP on network %ld (%s)\n",htonl(ipxs.sipx_network),
				SapOps[htons((int)*(short *)buf)]);
		size-=2;	/* header */
		size/=sizeof(SapRecord);
		sz=0;
		while(sz<size)
		{
			unsigned char *tp;
			int ct;
			switch(ntohs(s->Type))
			{
				case 0x03: printf("Print Queue   ");break;
				case 0x04: printf("File Server   ");break;
				case 0x05: printf("Job Server    ");break;
				case 0x07: printf("Print Server  ");break;
				case 0x09: printf("Archive Server");break;
				case 0x24: printf("Remote Bridge ");break;
				case 0x47: printf("Adv Print Srv ");break;
				case 0x4b: printf("Btrieve       ");break;
				case 0x98: printf("NACS Dialup   ");break;
				case 0x107:printf("Appletalk FS  ");break;
				default  : printf("Unknown(%04X) ",
						ntohs(s->Type));break;
			}
			printf("   %-48s\n",s->Name);
			printf("Network: %08X  Hops: %-4d  Socket: %04X  Node: ",
				ntohl(s->Network),ntohs(s->HopCount),
				ntohs(s->Socket));
			tp=s->Node;
			for(ct=0;ct<6;ct++)
				printf("%02X%s",*tp++,ct==5?"\n":":");
			sz++;
			s++;
		}
	}
}

